MODULE obs_field
   !!=====================================================================
   !!                       ***  MODULE obs_field  ***
   !! Observation diagnostics: Routines specific to model variables
   !!=====================================================================
   !! History : 4.0  !  2021-06  (D Ford)  Original code
   !!----------------------------------------------------------------------
   !!   obs_group_alloc   : allocate observation group types
   !!   obs_group_dealloc : deallocate observation group types
   !!   obs_group_check   : check observation group options
   !!----------------------------------------------------------------------

   !! * Modules used
   USE iom                ! In/out manager
   USE par_kind, ONLY : & ! Precision variables
      & wp
   USE obs_oper, ONLY : & ! Max number of daily avgd obs types
      & imaxavtypes
   USE obs_surf_def       ! Surface data definitions
   USE obs_profiles_def   ! Profile data definitions

   IMPLICIT NONE

   !! * Routine/type accessibility
   PRIVATE

   PUBLIC &
      & obs_group,         &
      & obs_group_alloc,   &
      & obs_group_dealloc, &
      & obs_group_read,    &
      & obs_group_check

   !! * Shared Module variables
   INTEGER, PARAMETER :: jpmaxntypes = 1000   ! Maximum number of obs types for each obs group
   INTEGER, PARAMETER :: jpmaxnfiles = 1000   ! Maximum number of files for each obs group
   
   ! Expected names for observation types with special behaviours (not needed for all observation types)
   CHARACTER(LEN=8), PUBLIC :: cobsname_uvel = 'UVEL' ! Expected variable name for U velocity (2D or 3D)
   CHARACTER(LEN=8), PUBLIC :: cobsname_vvel = 'VVEL' ! Expected variable name for V velocity (2D or 3D)
   CHARACTER(LEN=8), PUBLIC :: cobsname_sla  = 'SLA'  ! Expected variable name for SLA

   !! * Type definition for observation groups
   TYPE obs_group
      !
      CHARACTER(LEN=25)                             :: cgroupname    !: Name of obs group (for stdout)
      CHARACTER(LEN=8),   DIMENSION(:), ALLOCATABLE :: cobstypes     !: Observation types to read from files
      CHARACTER(LEN=128), DIMENSION(:), ALLOCATABLE :: cobsfiles     !: Observation file names
      CHARACTER(LEN=128), DIMENSION(:), ALLOCATABLE :: cobsbiasfiles !: Bias input file names
      CHARACTER(LEN=128)                            :: cbiasvarname  !: Bias variable name in input file
      CHARACTER(LEN=128)                            :: caltbiasfile  !: Altimeter bias input file name
      !
      INTEGER,            DIMENSION(:), ALLOCATABLE :: nprofdavtypes !: Profile data types representing a daily average
      !
      INTEGER  :: nobstypes          !: Number of observation types
      INTEGER  :: nobsfiles          !: Number of observation files
      INTEGER  :: nobsbiasfiles      !: Number of bias files
      INTEGER  :: nbiasvar           !: Index of observation type to be bias corrected
      INTEGER  :: navtypes           !: Number of profile data types representing a daily average
      INTEGER  :: nextvars           !: Number of extra variables in addition to any in input files
      INTEGER  :: naddvars           !: Number of additional variables in addition to any in input files
      INTEGER  :: n1dint             !: Type of vertical interpolation method
      INTEGER  :: n2dint             !: Type of horizontal interpolation method
      INTEGER  :: nmsshc             !: MSSH correction scheme
      INTEGER  :: nadd_ssh           !: Index of additional variable representing SSH
      INTEGER  :: next_mdt           !: Index of extra variable representing MDT
      !
      LOGICAL  :: lenabled           !: Logical switch for group being processed and not ignored
      LOGICAL  :: lsurf              !: Logical switch for surface data
      LOGICAL  :: lprof              !: Logical switch for profile data
      LOGICAL  :: lvel               !: Logical switch for velocity data
      LOGICAL  :: lsla               !: Logical switch for SLA data
      LOGICAL  :: laltbias           !: Logical switch for altimeter bias correction
      LOGICAL  :: lobsbias           !: Logical switch for bias correction
      LOGICAL  :: lnea               !: Logical switch for rejecting observations near land
      LOGICAL  :: lbound_reject      !: Logical switch for rejecting obs near the boundary
      LOGICAL  :: lignmis            !: Logical switch for ignoring missing files
      LOGICAL  :: lall_at_all        !: Logical switch for computing all model variables at all obs points
      LOGICAL  :: lnight             !: Logical switch for calculating night-time average
      LOGICAL  :: lfp_indegs         !: Logical: T=> averaging footprint is in degrees, F=> in metres
      !
      REAL(wp) :: ravglamscl         !: E/W diameter of observation footprint (metres/degrees)
      REAL(wp) :: ravgphiscl         !: N/S diameter of observation footprint (metres/degrees)
      REAL(wp) :: rmdtcorr           !: MDT correction
      REAL(wp) :: rmdtcutoff         !: MDT cutoff for computed correction
      !
      REAL(wp), POINTER, DIMENSION(:,:,:)   :: rglam  !: Longitudes
      REAL(wp), POINTER, DIMENSION(:,:,:)   :: rgphi  !: Latitudes
      REAL(wp), POINTER, DIMENSION(:,:,:,:) :: rmask  !: Land/sea masks
      !
      TYPE(obs_surf) :: ssurfdata    !: Initial surface data
      TYPE(obs_surf) :: ssurfdataqc  !: Surface data after quality control
      TYPE(obs_prof) :: sprofdata    !: Initial profile data
      TYPE(obs_prof) :: sprofdataqc  !: Profile data after quality control
      !
   END TYPE

CONTAINS

   SUBROUTINE obs_group_alloc( sdobsgroup )
      !!----------------------------------------------------------------------
      !!                     ***  ROUTINE obs_group_alloc  ***
      !!
      !! ** Purpose : - Allocate data for observation group types
      !!
      !! ** Method  : - Allocate arrays
      !!
      !! ** Action  : - Allocate arrays
      !!
      !!----------------------------------------------------------------------
      !! * Arguments
      TYPE(obs_group), INTENT(INOUT) :: sdobsgroup ! Obs group to be allocated
      !!----------------------------------------------------------------------

      ALLOCATE( sdobsgroup%cobstypes    (sdobsgroup%nobstypes            ), &
         &      sdobsgroup%cobsfiles    (sdobsgroup%nobsfiles            ), &
         &      sdobsgroup%cobsbiasfiles(sdobsgroup%nobsbiasfiles        ), &
         &      sdobsgroup%nprofdavtypes(sdobsgroup%navtypes             ), &
         &      sdobsgroup%rglam        (jpi,jpj,    sdobsgroup%nobstypes), &
         &      sdobsgroup%rgphi        (jpi,jpj,    sdobsgroup%nobstypes), &
         &      sdobsgroup%rmask        (jpi,jpj,jpk,sdobsgroup%nobstypes) )

   END SUBROUTINE obs_group_alloc


   SUBROUTINE obs_group_dealloc( sdobsgroup )
      !!----------------------------------------------------------------------
      !!                     ***  ROUTINE obs_group_dealloc  ***
      !!
      !! ** Purpose : - Deallocate data for observation group types
      !!
      !! ** Method  : - Deallocate arrays
      !!
      !! ** Action  : - Deallocate arrays
      !!
      !!----------------------------------------------------------------------
      !! * Arguments
      TYPE(obs_group), INTENT(INOUT) :: sdobsgroup ! Obs group to be deallocated
      !!----------------------------------------------------------------------

      DEALLOCATE( sdobsgroup%cobstypes,     &
         &        sdobsgroup%cobsfiles,     &
         &        sdobsgroup%cobsbiasfiles, &
         &        sdobsgroup%nprofdavtypes, &
         &        sdobsgroup%rglam,         &
         &        sdobsgroup%rgphi,         &
         &        sdobsgroup%rmask )

   END SUBROUTINE obs_group_dealloc


   SUBROUTINE obs_group_read( sdobsgroup )
      !!----------------------------------------------------------------------
      !!                     ***  ROUTINE obs_group_read  ***
      !!
      !! ** Purpose : - Read namelist for observation group types
      !!
      !! ** Method  : - Read namelist
      !!
      !! ** Action  : - Read namelist
      !!
      !!----------------------------------------------------------------------
      !! * Arguments
      TYPE(obs_group), INTENT(INOUT) :: sdobsgroup ! Obs group to be populated
      !! * Local variables
      INTEGER :: ios                               ! Status for namelist read
      INTEGER :: itype, ifile                      ! Loop counters
      INTEGER :: jtype, jfile                      ! Loop counters
      !
      CHARACTER(LEN=128)                         :: cn_groupname
      CHARACTER(LEN=8),   DIMENSION(jpmaxntypes) :: cn_obstypes
      CHARACTER(LEN=128), DIMENSION(jpmaxnfiles) :: cn_obsfiles
      CHARACTER(LEN=128), DIMENSION(jpmaxnfiles) :: cn_obsbiasfiles
      CHARACTER(LEN=128)                         :: cn_type_to_biascorrect
      CHARACTER(LEN=128)                         :: cn_obsbiasfile_varname
      CHARACTER(LEN=128)                         :: cn_altbiasfile
      INTEGER,            DIMENSION(imaxavtypes) :: nn_profdavtypes
      INTEGER                                    :: nn_1dint
      INTEGER                                    :: nn_2dint
      INTEGER                                    :: nn_msshc
      LOGICAL                                    :: ln_enabled
      LOGICAL                                    :: ln_surf
      LOGICAL                                    :: ln_prof
      LOGICAL                                    :: ln_altbias
      LOGICAL                                    :: ln_obsbias
      LOGICAL                                    :: ln_nea
      LOGICAL                                    :: ln_bound_reject
      LOGICAL                                    :: ln_ignmis
      LOGICAL                                    :: ln_all_at_all
      LOGICAL                                    :: ln_night
      LOGICAL                                    :: ln_fp_indegs
      REAL(wp)                                   :: rn_avglamscl
      REAL(wp)                                   :: rn_avgphiscl
      REAL(wp)                                   :: rn_mdtcorr
      REAL(wp)                                   :: rn_mdtcutoff
      !!
      NAMELIST/namobs_dta/cn_groupname, ln_prof, ln_surf, ln_enabled,           &
         &                cn_obsfiles, cn_obstypes, ln_nea, ln_bound_reject,    &
         &                ln_ignmis, nn_2dint, nn_1dint, nn_profdavtypes,       &
         &                ln_fp_indegs, rn_avglamscl, rn_avgphiscl, ln_obsbias, &
         &                cn_obsbiasfiles, cn_type_to_biascorrect,              &
         &                cn_obsbiasfile_varname, ln_night, ln_altbias,         &
         &                cn_altbiasfile, nn_msshc, rn_mdtcorr, rn_mdtcutoff,   &
         &                ln_all_at_all
      !!----------------------------------------------------------------------

      cn_obstypes(:)     = ''
      cn_obsfiles(:)     = ''
      cn_obsbiasfiles(:) = ''
      nn_profdavtypes(:) = -1

      ! There is only one namobs_dta block in namelist_ref -> use it for each group so we do a rewind
      REWIND(numnam_ref)
      READ  ( numnam_ref, namobs_dta, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 )   CALL ctl_nam ( ios , 'namobs_dta in reference namelist' )

      ! Read namobs_dta from namelist_cfg
      ! WARNING: we don't do a rewind here, each group reads its own namobs_dta block one after another
      READ  ( numnam_cfg, namobs_dta, IOSTAT = ios, ERR = 902 )
902   IF( ios >  0 )   CALL ctl_nam ( ios , 'namobs_dta in configuration namelist' )
      IF(lwm) WRITE( numond, namobs_dta )
      
      sdobsgroup%cgroupname = cn_groupname
      sdobsgroup%lenabled   = ln_enabled

      IF (sdobsgroup%lenabled) THEN
         sdobsgroup%nobstypes     = 0
         sdobsgroup%nobsfiles     = 0
         sdobsgroup%naddvars      = 0
         sdobsgroup%nextvars      = 0
         sdobsgroup%navtypes      = 0
         sdobsgroup%nobsbiasfiles = 0
         sdobsgroup%lvel          = .false.
         sdobsgroup%lsla          = .false.
         sdobsgroup%nadd_ssh      = 0
         sdobsgroup%next_mdt      = 0

         DO jtype = 1, jpmaxntypes
            IF ( TRIM(cn_obstypes(jtype)) /= '' ) THEN
               sdobsgroup%nobstypes = sdobsgroup%nobstypes + 1
            ENDIF
         END DO
         DO jfile = 1, jpmaxnfiles
            IF ( TRIM(cn_obsfiles(jfile)) /= '' ) THEN
               sdobsgroup%nobsfiles = sdobsgroup%nobsfiles + 1
            ENDIF
         END DO
         DO jtype = 1, imaxavtypes
            IF ( nn_profdavtypes(jtype) /= -1 ) THEN
               sdobsgroup%navtypes = sdobsgroup%navtypes + 1
            ENDIF
         END DO
         DO jfile = 1, jpmaxnfiles
            IF ( TRIM(cn_obsbiasfiles(jfile)) /= '' ) THEN
               sdobsgroup%nobsbiasfiles = sdobsgroup%nobsbiasfiles + 1
            ENDIF
         END DO

         CALL obs_group_alloc( sdobsgroup )

         itype = 0
         DO jtype = 1, jpmaxntypes
            IF ( TRIM(cn_obstypes(jtype)) /= '' ) THEN
               itype = itype + 1
               sdobsgroup%cobstypes(itype) = TRIM(cn_obstypes(jtype))
               IF ( (TRIM(sdobsgroup%cobstypes(itype)) == cobsname_uvel) .OR. &
                  & (TRIM(sdobsgroup%cobstypes(itype)) == cobsname_vvel) ) THEN
                  sdobsgroup%lvel = .true.
               ELSEIF ( TRIM(sdobsgroup%cobstypes(itype)) == cobsname_sla ) THEN
                  sdobsgroup%lsla = .true.
                  ! SSH=additional, MDT=extra
                  sdobsgroup%naddvars = sdobsgroup%naddvars + 1
                  sdobsgroup%nextvars = sdobsgroup%nextvars + 1
                  sdobsgroup%nadd_ssh = sdobsgroup%naddvars
                  sdobsgroup%next_mdt = sdobsgroup%nextvars
! DO THIS FOR FBD TOO
               ENDIF
               !
               IF (TRIM(sdobsgroup%cobstypes(itype)) == cobsname_uvel) THEN
                  sdobsgroup%rglam(:,:,itype)   = glamu(:,:)
                  sdobsgroup%rgphi(:,:,itype)   = gphiu(:,:)
                  sdobsgroup%rmask(:,:,:,itype) = umask(:,:,:)
               ELSEIF (TRIM(sdobsgroup%cobstypes(itype)) == cobsname_vvel) THEN
                  sdobsgroup%rglam(:,:,itype)   = glamv(:,:)
                  sdobsgroup%rgphi(:,:,itype)   = gphiv(:,:)
                  sdobsgroup%rmask(:,:,:,itype) = vmask(:,:,:)
               ELSE
                  sdobsgroup%rglam(:,:,itype)   = glamt(:,:)
                  sdobsgroup%rgphi(:,:,itype)   = gphit(:,:)
                  sdobsgroup%rmask(:,:,:,itype) = tmask(:,:,:)
               ENDIF
            ENDIF
         END DO
         ifile = 0
         DO jfile = 1, jpmaxnfiles
            IF ( TRIM(cn_obsfiles(jfile)) /= '' ) THEN
               ifile = ifile + 1
               sdobsgroup%cobsfiles(ifile) = TRIM(cn_obsfiles(jfile))
            ENDIF
         END DO
         itype = 0
         DO jtype = 1, imaxavtypes
            IF ( nn_profdavtypes(jtype) /= -1 ) THEN
               itype = itype + 1
               sdobsgroup%nprofdavtypes(itype) = nn_profdavtypes(jtype)
            ENDIF
         END DO
         ifile = 0
         DO jfile = 1, jpmaxnfiles
            IF ( TRIM(cn_obsbiasfiles(jfile)) /= '' ) THEN
               ifile = ifile + 1
               sdobsgroup%cobsbiasfiles(ifile) = cn_obsbiasfiles(jfile)
            ENDIF
         END DO
         IF ( ln_obsbias ) THEN
            sdobsgroup%nbiasvar = -1
            DO jtype = 1, sdobsgroup%nobstypes
               IF ( TRIM(sdobsgroup%cobstypes(itype)) == TRIM(cn_type_to_biascorrect) ) THEN
                  sdobsgroup%nbiasvar = jtype
                  EXIT
               ENDIF
            ENDDO
         ENDIF

         sdobsgroup%caltbiasfile       = cn_altbiasfile
         sdobsgroup%n1dint             = nn_1dint
         sdobsgroup%n2dint             = nn_2dint
         sdobsgroup%nmsshc             = nn_msshc
         sdobsgroup%lsurf              = ln_surf
         sdobsgroup%lprof              = ln_prof
         sdobsgroup%laltbias           = ln_altbias
         sdobsgroup%lobsbias           = ln_obsbias
         sdobsgroup%cbiasvarname       = cn_obsbiasfile_varname
         sdobsgroup%lnea               = ln_nea
         sdobsgroup%lbound_reject      = ln_bound_reject
         sdobsgroup%lignmis            = ln_ignmis
         sdobsgroup%lall_at_all        = ln_all_at_all
         sdobsgroup%lnight             = ln_night
         sdobsgroup%lfp_indegs         = ln_fp_indegs
         sdobsgroup%ravglamscl         = rn_avglamscl
         sdobsgroup%ravgphiscl         = rn_avgphiscl
         sdobsgroup%rmdtcorr           = rn_mdtcorr
         sdobsgroup%rmdtcutoff         = rn_mdtcutoff
      ENDIF

   END SUBROUTINE obs_group_read


   SUBROUTINE obs_group_check( sdobsgroup, kgroup )
      !!----------------------------------------------------------------------
      !!                     ***  ROUTINE obs_group_check  ***
      !!
      !! ** Purpose : - Error check observation group types
      !!
      !! ** Method  : - Check and print options
      !!
      !! ** Action  : - Check and print options
      !!
      !!----------------------------------------------------------------------
      !! * Arguments
      TYPE(obs_group), INTENT(IN) :: sdobsgroup ! Obs group to be checked
      INTEGER,         INTENT(IN) :: kgroup     ! Number of group being checked
      !! * Local variables
      INTEGER :: jtype, jfile                   ! Loop counters
      !!----------------------------------------------------------------------

      IF (lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'obs_group_check : Options for group ', kgroup, ', ', TRIM(sdobsgroup%cgroupname)
         WRITE(numout,*) '~~~~~~~~~~~~'
         WRITE(numout,*) '          Logical switch for group being enabled                  ln_enabled = ', sdobsgroup%lenabled
         IF ( .NOT. sdobsgroup%lenabled ) THEN
            WRITE(numout,*) '             Group disabled, will not be used'
         ELSE
            WRITE(numout,*) '          Observation types in group:', sdobsgroup%nobstypes
            DO jtype = 1, sdobsgroup%nobstypes
               WRITE(numout,*) '             ', TRIM(sdobsgroup%cobstypes(jtype))
            END DO
               WRITE(numout,*) '          Observation files in group:', sdobsgroup%nobsfiles
            DO jfile = 1, sdobsgroup%nobsfiles
               WRITE(numout,*) '             ', TRIM(sdobsgroup%cobsfiles(jfile))
            END DO
            WRITE(numout,*) '          General settings:'
            WRITE(numout,*) '             Logical switch for surface data                         ln_surf = ', sdobsgroup%lsurf
            WRITE(numout,*) '             Logical switch for profile data                         ln_prof = ', sdobsgroup%lprof
            WRITE(numout,*) '             Rejection of observations near land switch               ln_nea = ', sdobsgroup%lnea
            WRITE(numout,*) '             Rejection of obs near open bdys                 ln_bound_reject = ', sdobsgroup%lbound_reject
            WRITE(numout,*) '             Logical switch for ignoring missing files             ln_ignmis = ', sdobsgroup%lignmis
            WRITE(numout,*) '             Type of horizontal interpolation method                nn_2dint = ', sdobsgroup%n2dint
            IF ( sdobsgroup%n2dint <= 4 ) THEN
               WRITE(numout,*) '                model counterparts will be interpolated horizontally'
            ELSE
               WRITE(numout,*) '                model counterparts will be averaged horizontally'
            ENDIF
            WRITE(numout,*) '          Settings only for surface data, which is ', sdobsgroup%lsurf
            WRITE(numout,*) '             Obs footprint in deg [T] or m [F]                  ln_fp_indegs = ', sdobsgroup%lfp_indegs
            WRITE(numout,*) '             E/W diameter of obs footprint                      rn_avglamscl = ', sdobsgroup%ravglamscl
            WRITE(numout,*) '             N/S diameter of obs footprint                      rn_avgphiscl = ', sdobsgroup%ravgphiscl
            WRITE(numout,*) '             Logical switch for night-time average                  ln_night = ', sdobsgroup%lnight
            WRITE(numout,*) '             Logical switch for bias correction                   ln_obsbias = ', sdobsgroup%lobsbias
            IF ( sdobsgroup%lobsbias ) THEN
               WRITE(numout,*) '             Observation type to be bias corrected    cn_type_to_biascorrect = ', TRIM(sdobsgroup%cobstypes(sdobsgroup%nbiasvar))
               WRITE(numout,*) '             Bias variable name in bias files         cn_obsbiasfile_varname = ', TRIM(sdobsgroup%cbiasvarname)
               WRITE(numout,*) '             Bias files in group:', sdobsgroup%nobsbiasfiles
               DO jfile = 1, sdobsgroup%nobsbiasfiles
                  WRITE(numout,*) '                ', TRIM(sdobsgroup%cobsbiasfiles(jfile))
               END DO
            ENDIF
            WRITE(numout,*) '          Settings only for profile data, which is ', sdobsgroup%lprof
            WRITE(numout,*) '             Type of vertical interpolation method                  nn_1dint = ', sdobsgroup%n1dint
            WRITE(numout,*) '             Daily average types                             nn_profdavtypes = ', sdobsgroup%nprofdavtypes
            WRITE(numout,*) '             Logical switch to compute all vars at all pts     ln_all_at_all = ', sdobsgroup%lall_at_all
            WRITE(numout,*) '          Settings only for SLA data, which is ', sdobsgroup%lsla
            WRITE(numout,*) '             Logical switch for alt bias                          ln_altbias = ', sdobsgroup%laltbias
            WRITE(numout,*) '             Alt bias file name                               cn_altbiasfile = ', TRIM(sdobsgroup%caltbiasfile)
            WRITE(numout,*) '             MSSH correction scheme                                 nn_msshc = ', sdobsgroup%nmsshc
            WRITE(numout,*) '             MDT  correction                                      rn_mdtcorr = ', sdobsgroup%rmdtcorr
            WRITE(numout,*) '             MDT cutoff for computed correction                 rn_mdtcutoff = ', sdobsgroup%rmdtcutoff
         ENDIF

         IF ( (       sdobsgroup%lsurf .AND.       sdobsgroup%lprof ) .OR. &
            & ( .NOT. sdobsgroup%lsurf .AND. .NOT. sdobsgroup%lprof ) ) THEN
            CALL ctl_stop( ' One and only one of ln_surf or ln_prof must be set per observation group' )
         ENDIF

         IF ( sdobsgroup%nobstypes == 0 ) THEN
            CALL ctl_stop( ' No observation types specified for this observation group' )
         ENDIF

         IF ( sdobsgroup%nobsfiles == 0 ) THEN
            CALL ctl_stop( ' No observation files specified for this observation group' )
         ENDIF

         IF ( (sdobsgroup%lobsbias) .AND. (sdobsgroup%nobsbiasfiles == 0) ) THEN
            CALL ctl_stop( ' No bias files specified for this observation group' )
         ENDIF

         IF ( (sdobsgroup%n2dint < 0) .OR. (sdobsgroup%n2dint > 6) ) THEN
            CALL ctl_stop( ' Invalid horizontal interpolation type' )
         ENDIF

         IF( (sdobsgroup%n1dint < 0) .OR. (sdobsgroup%n1dint > 1) ) THEN
            CALL ctl_stop(' Invalid vertical interpolation type')
         ENDIF

         IF ( (sdobsgroup%n2dint > 4) .AND. (sdobsgroup%n2dint <= 6) ) THEN
            IF ( sdobsgroup%ravglamscl <= 0._wp ) THEN
               CALL ctl_stop( ' Incorrect value set for averaging footprint scale rn_avglamscl' )
            ENDIF
            IF ( sdobsgroup%ravgphiscl <= 0._wp ) THEN
               CALL ctl_stop( ' Incorrect value set for averaging footprint scale rn_avgphiscl' )
            ENDIF
         ENDIF
         
         ! Various other checks of combined options

      ENDIF

   END SUBROUTINE obs_group_check

END MODULE obs_field
